/*
 * PM 2018
 * Nicolae State 333CC 
 * Fruit Piano
 */
 
#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include "CapacitiveSensor.h"


#define CAP_THRESHOLD	150
#define STEPS		10
#define NUM_NOTES		8
#define SOME_HARDCODED_VALUE 50

#define DO		262
#define RE		294
#define MI		330 
#define FA		349
#define SOL		392
#define LA		440
#define SI		494
#define DO2		523

int frequencies[NUM_NOTES] = {
	DO, RE, MI, FA, SOL, LA, SI, DO2
};

// Initialize the sensor with the recvPins
// The sendPin is PC6 by default
CapacitiveSensor notes[] = {
	CapacitiveSensor(PA0),
	CapacitiveSensor(PA1),
	CapacitiveSensor(PA2),
	CapacitiveSensor(PA3),
	CapacitiveSensor(PA4),
	CapacitiveSensor(PA5),
	CapacitiveSensor(PA6),
	CapacitiveSensor(PA7),
};

// Needed for the time check between sendPin and recvPin
volatile unsigned long currentMillis = 0;

// Make the buzzer play a given note
void play_note(int note) {
	int period = 1000000l / frequencies[note];
	int i;

	PORTB |= (1 << PB3);
	for (i = 0; i < period / 2; ++i)
		_delay_us(1);

	PORTB &= ~(1 << PB3);
	for (i = 0; i < period / 2; ++i)
		_delay_us(1);
}

void init() 
{
	// Turn off re-calibration
	for(int i = 0; i < NUM_NOTES ; ++i)
	{
		notes[i].set_CS_AutocaL_Millis(0xFFFFFFFF);
	}
	
	// Set LED as output (for testing)
	DDRD |= (1 << PD7);
	
	// Set BUZZER as output
	DDRB |= (1 << PB3);
	PORTB &= ~(1 << PB3);
	
	//PC6  - output - (sendPin)
	DDRC |= (1 << PC6);
	
	// PA's - input - (recvPins) -> Fruits
	DDRA &= ~(1 << PA0);
	DDRA &= ~(1 << PA1);
	DDRA &= ~(1 << PA2);
	DDRA &= ~(1 << PA3);
	DDRA &= ~(1 << PA4);
	DDRA &= ~(1 << PA5);
	DDRA &= ~(1 << PA6);
	DDRA &= ~(1 << PA7);

}

int main()
{
	init();
	int i = 0;
	int avg = 0;

	while(1)
    {
		// Check all the buttons
		for (int button = 0; button < NUM_NOTES; button ++)
		{
			// Get an average noise factor (10 steps)
			for(i = 0; i < STEPS ; i ++)
			{
				avg += notes[button].capacitiveSensor(1);
			}
			avg = avg / STEPS;

			// Check touching detection (cap changed)
			long sensor = avg;
			while(sensor > CAP_THRESHOLD)
			{
				// Light LED
				PORTD |= (1 << PD7);
				
				// Stabilize the sound
				for(i = 0 ; i < SOME_HARDCODED_VALUE; i ++)
				{
						play_note(button);
				}
				
				// Get the average noise factor inside the loop
				for(i = 0; i < STEPS * 2 ; i ++)
				{
					avg += notes[button].capacitiveSensor(1);
				}

				avg = avg / STEPS;
				sensor = avg;
			}
			
			// When here, fruits are not touched
			
			// Shut down LED
			PORTD &= ~(1 << PD7);

		}
    }
}